package bank.client.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import bank.common.Account;
import bank.common.Bank;
import bank.common.Customer;
import bank.common.RemoteAccountEvent;
import bank.common.RemoteAccountListener;

public class BankClientGUI {

	private final ExecutorService ex = Executors.newCachedThreadPool();

	private JFrame frame;
	private JTextField amountField;
	private JButton depositBtn;
	private JButton withdrawBtn;
	private JTextArea balanceTextA;

	private Registry reg;
	private Bank bank;
	private Account account;
	private AccountWatcher accountWatcher;

	public static void main(String[] args) {
		BankClientGUI ui;
		try {
			ui = new BankClientGUI();
			ui.open();
		} catch (RemoteException | NotBoundException e) {
			System.out.println("Unable to access server ");
			e.printStackTrace();
		}
	}

	private BankClientGUI() throws RemoteException, NotBoundException {
		reg = LocateRegistry.getRegistry("localhost", Bank.PORT);
		bank = (Bank) reg.lookup("Bank");

		String name = JOptionPane.showInputDialog("Please input customer name");
		Customer customer = bank.getCustomer(name);
		if (customer == null) {
			customer = bank.createCustomer(name);
		}
		Account[] accounts = customer.getAccounts();
		if (accounts.length == 0) {
			account = bank.createAccount(name);
		} else {
			account = accounts[0];
		}
		frame = new JFrame("Bank Client for " + name + " Account " + account.getId());
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		Container cp = frame.getContentPane();
		cp.setLayout(new BorderLayout());
		JPanel btPanel = new JPanel();
		cp.add(btPanel, BorderLayout.NORTH);
		depositBtn = new JButton("Deposit");
		withdrawBtn = new JButton("Withdraw");
		amountField = new JTextField(20);
		amountField.setText("");
		btPanel.add(new JLabel("Amount: "));
		btPanel.add(amountField);
		btPanel.add(depositBtn);
		btPanel.add(withdrawBtn);
		balanceTextA = new JTextArea(2, 20);
		balanceTextA.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		cp.add(balanceTextA, BorderLayout.CENTER);
		balanceTextA.setText("Balance = " + account.getBalance());

		depositBtn.addActionListener(a -> {
			ex.submit(() -> {
				try {
					long amount = Long.parseLong(amountField.getText());
					account.deposit(amount);
				} catch (Exception e) {
					System.out.println("Unable to deposition on account ");
					e.printStackTrace();
				}
			});
		});

		withdrawBtn.addActionListener(a -> {
			try {
				long amount = Long.parseLong(amountField.getText());
				account.withdraw(amount);
			} catch (Exception e) {
				System.out.println("Unable to deposition on account ");
				e.printStackTrace();
			}
		});

		accountWatcher = new AccountWatcher(account);
		account.addRemoteAccountListener(accountWatcher);

		frame.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				try {
					account.removeRemoteAccountListener(accountWatcher);
					UnicastRemoteObject.unexportObject(accountWatcher, true);
				} catch (RemoteException e1) {
					e1.printStackTrace();
				}

				super.windowClosing(e);
			}

		});
	}

	private void open() {
		SwingUtilities.invokeLater(() -> {
			depositBtn.setEnabled(true);
			withdrawBtn.setEnabled(true);

			frame.pack();
			frame.setLocation(200, 200);
			frame.setVisible(true);
		});
	}

	@SuppressWarnings("serial")
	private class AccountWatcher extends UnicastRemoteObject implements RemoteAccountListener {
		@SuppressWarnings("unused")
		private final Account account;

		public AccountWatcher(Account account) throws RemoteException {
			super();
			this.account = account;
		}

		@Override
		public void accountChanged(RemoteAccountEvent e) throws RemoteException {
			balanceTextA.setText("Balance = " + e.getBalance());
		}

	};

}
